#include "Mix3FilterLATE.h"
#include "PluginPipeSettings.h"
#include "PluginTile.h"
#include "PluginImage.h"
#include "PluginImageSettings.h"
#include "PluginOptionList.h"

#include "Mix3Plugin.h"
#include "Mix3Loop.h"
#include "cmath"

#include <QDebug>
#include <iostream>

using namespace std;

//	Check if the plugin needs to run
bool Mix3FilterLATE::needsToRunLayer(const PluginImageSettings &options, const PluginPipeSettings &settings, PluginOptionList &layerOptions) const
{
	Q_UNUSED( options );
	Q_UNUSED( settings );
	
	bool okay;
        bool enabl = layerOptions.getBool(ID_MIX3_ENABLED, m_groupId, okay); //is the plugin enabled?
        bool val = layerOptions.getBool(ID_MIX3_EARLY, m_groupId, okay); //is the checkbox checked?
	return enabl && !val; //run if the plugin is enabled and the checkbox isn't checked
}

QTransform Mix3FilterLATE::transform(const PluginImageSettings &options, const PluginPipeSettings &settings) const
{
	Q_UNUSED( options );
	Q_UNUSED( settings );
	
	return QTransform();	//	We don't warp the image so return an identity transform.
}

QSize Mix3FilterLATE::size(const PluginImageSettings &options, const PluginPipeSettings &settings, const QSize &startSize) const
{
	Q_UNUSED( options );
	Q_UNUSED( settings );
	
	return startSize;	//	We don't change the size of the image.
}

QList<PluginDependency*> Mix3FilterLATE::prerequisites(const PluginImageSettings &options, const PluginPipeSettings &settings, PluginTile &tile) const
{
	Q_UNUSED( options );
	Q_UNUSED( settings );
	Q_UNUSED( tile );
	
	return QList<PluginDependency*>();	//	Empty list - we don't depend on any other tiles.
}

//	******************************************************************************
//	The processing function
//	Basically very similar to the B5 version, the processing loop can almost be used directly.
//	******************************************************************************
void Mix3FilterLATE::runLayer(const PluginImageSettings &options, const PluginPipeSettings &settings, PluginTile &tile, PluginOptionList &layerOptions, int layerPos) const
{
	Q_UNUSED( options );
	Q_UNUSED( settings );
	Q_UNUSED( layerOptions );
	
	PluginImage *	pSrcImg = tile.image();
	PluginImage *	pDstImg;
	
	//	Layer 0 output should always go into tile.image(), otherwise output should go into tile.tmpImage()
	if ( layerPos == 0 )
		pDstImg = tile.image();
	else
		pDstImg = tile.tmpImage();

	if((pSrcImg == NULL) || (pDstImg == NULL)) return; //NULLPointer check for safety
	
	//	Tile parameters
	int rw = pSrcImg->rowWords();	//	number of unsigned shorts in a row 
	int pw = pSrcImg->planeWords();	//	increment to get to the next color component in the plane
    int np = pSrcImg->nextPixel();	//	increment for the next pixel within the same color plane

	int tileWidth = settings.paddedWidth();        //Tile Dimensions
	int tileHeight = settings.paddedHeight();

	//	Reading the options
    bool bOk;
	int expo,rr,gg,bb;
	float R,G,B,e,r,g,b,sumrgb;

    expo = layerOptions.getInt( ID_MIX3_EXPOSURE, m_groupId, bOk );
	rr   = layerOptions.getInt( ID_MIX3_RED, m_groupId, bOk );
	gg   = layerOptions.getInt( ID_MIX3_GREEN, m_groupId, bOk );
	bb   = layerOptions.getInt( ID_MIX3_BLUE, m_groupId, bOk );

    e = expo / 100.f;
    r = 1.0f + (rr / 100.f);
    g = 1.0f + (gg / 100.f);
    b = 1.0f + (bb / 100.f);

    if ((r+g+b)==0.0f)
    {
        sumrgb=1.0f;
    }
    else
    {
        sumrgb = r+g+b;
    }
   
	uint16 *pIn = NULL;
	uint16 *pOut = NULL;

    //	Processing loop
    for ( int y = 0; y < tileHeight; y++ )
	{
		pIn = pSrcImg->data16() + rw * y;
		pOut = pDstImg->data16() + rw * y;

		for ( int x = 0; x < tileWidth; x++ )
		{

			// ********START MY LOOP*******

            // Modifications for HSV approach - Note that LUT does 65535
         	R = ((float)*(pIn));
            G = ((float)*(pIn + pw));
            B = ((float)*(pIn + 2*pw));

            // All the mathematical calculations are performed in this routine
            Mix3Loop( expo,  e,  R,  G,  B,  r,  g,  b, sumrgb);

     		// Ensure the boundaries are not breached
            R=(R>65535)?65535:R;
            R=(R<0)    ?0    :R;
            G = R;
            B = R;

			*(pOut)=(int)R;
			*(pOut+pw)=(int)G;
			*(pOut+pw+pw)=(int)B;

            //***********END MY LOOP*******

			pIn+=np;pOut+=np;
		}
	}
}